clear; close all; clc; format long
EPMAdata = load('EPMAdata')
counter = 0 ;
Baker = 0 ; % 0 for Baker, anything else: Mucci (1987)

%% Rmod can be given multiple inputs for multiple model runs.
for Rmod = [1] % Rmod us the reciprocal (units of years) of the reaction rate.
sys.nX = 100 ; % Number of cells in model

sys.X = linspace(min(EPMAdata.x),max(EPMAdata.x),sys.nX)
sys.dX = zeros(1,sys.nX)+abs(sys.X(1)-sys.X(2)); 

sys.phi = 10^-6 ; % modeled porosity of intra-crystalline water films

%% sys structure: for parameters for running the model
sys.R = @(t)  1 / (Rmod * 3.154 * 10^7) ; % Calculate reaction rate (in s-1)

%% EQs structure: equations for constructing and running model
% Water Information
EQs.PW = 1.03; % Density of water
EQs.AMU.W = 18 ; % AMU of water;

% Carbonate Information
EQs.PC = 2.7; % density of LMC, g/cc

EQs.AMU.C = 100; % AMU of carbonate, g/cc

% Chemistry equations
EQs.alphaO = @(T) exp(( 18030./(T+273.15) - 32.24 ) ./ 1000 ); % Alpha-value for oxygen (Kim and O'Neil, 1997)
EQs.alphaC = @(T) 1.001 ; % Alpha-value for Carbon (Romanek et al., 1992)
if Baker==1
    EQs.Kd.Mg = @(T) 8.1 * 10^-4; % From Baker et al., (1982)
else
    EQs.Kd.Mg = @(T) 0.012 ; % Distribution coefficient for Magnesium Mucci (1987)
end
EQs.Kd.Sr = @(T) 0.05 ; % Distribution coefficient for Strontium Banner (1995) and Banner and Hanson (1990).

% Diffusion coefficients from Boudreu (1997)
EQs.Di.Ca = @(T,phi) (3.60 + 0.144 .* T)*10^-6 ./ (1-log(phi.^2)) ; % Diffusivity of Calcium (T in °C)
EQs.Di.Mg = @(T,phi) (3.43 + 0.144 .* T)*10^-6 ./ (1-log(phi.^2)) ; % Diffusivity of Magnesium
EQs.Di.Sr = @(T,phi) (3.69 + 0.169 .* T)*10^-6 ./ (1-log(phi.^2)) ; % Diffusivity of Strontium
EQs.Di.C  = @(T,phi) (5.06 + 0.275 .* T)*10^-6 ./ (1-log(phi.^2)) ; % Diffusivity of Bicarbonate
EQs.Di.O  = @(T,phi) (8.702 + 0.443 * T + 0.0035 * T^2) * 10^-6 ./ (1-log(phi.^2)) ; 

EQs.ROsmow = 0.0020052 ; % Baertschi (1976)
EQs.ROpdb = EQs.ROsmow * 1.03092; % PDB Isotope Ratio using SMOW values from Baertschi (1976) and PDB = 1.03092*SMOW from Coplen et al., (1983)
EQs.RCpdb = 0.01118; % Chang and Li (1990)

%% External Boundary Condition
%  Functions in BC are called by time, so they can vary over time.
% Fluid BC values should be in units of mols / g
BC.d18O = @(t) -0.89 ; 
BC.DIC = @(t) 1 * 10^-6; % mols / g solvent
BC.d13C = @(t) -0.8;

BC.Ca   = @(t) (16 * 10^-6); % Concentration of seawater Ca (mol/g), Horita et al., (2002)
BC.MgCa = @(t) 2.2;  % Eocene Mg abundance Evans et al., (2018)
BC.SrCa = @(t) 8/1000 ; % Eocene Sr Abundance Lear et al., (2003)

% Calculate things based on above parameters
BC.O6 = @(t) (1 / EQs.AMU.W) * (1 / (1 + (BC.d18O(t)/1000 + 1)*EQs.ROsmow)) ; % Concentration of 16O (mols/g solvent)
BC.O8 = @(t) BC.O6(t) * EQs.ROsmow * (1 + BC.d18O(t)/1000) ; % Concentration of 18O (mols/g solvent)

BC.C2 = @(t) BC.DIC(t) * (1 / (1 + (BC.d13C(t)/1000 + 1) * EQs.RCpdb)); % Concentration of 13C (mols/g solvent)
BC.C3 = @(t) BC.DIC(t) - BC.C2(t); % Concentration of 12C (mols/g solvent)

BC.Mg = @(t) BC.Ca(t) * BC.MgCa(t); % Concentration of Mg (mols/g solvent)
BC.Sr = @(t) BC.Ca(t) * BC.SrCa(t); % Concentration of Sr (mols/g solvent)

BC.T = @(t) 10; % temperature (°C)


%% INIT structure: information on initial state of model
INIT.MgCa = interp1(EPMAdata.x,EPMAdata.MgCa,sys.X) /1000; 

INIT.SrCa = interp1(EPMAdata.x,EPMAdata.SrCa,sys.X) /1000;

INIT.d18O = zeros(1,sys.nX) - 3.49 ; 

INIT.d13C = zeros(1,sys.nX) + 3.33


%% Construct initial vector, V0
c = 1; % c is a counter, keeping track of elements in the initial vector as they are added

MolConCarb = 1/EQs.AMU.C ;  % molar Concentration of Carbonate/g of CaCO3

for ii = 1:sys.nX
    %% Pore Water 
    % Initial porewater is the same as the outer boundary condition at t=0
    V0(c) = BC.Ca(0) ; ID.PW.Ca(ii) = c ; c=c+1 ;
    V0(c) = BC.Mg(0) ; ID.PW.Mg(ii) = c ; c=c+1 ;
    V0(c) = BC.Sr(0) ; ID.PW.Sr(ii) = c ; c=c+1 ;
    V0(c) = BC.O6(0) ; ID.PW.O6(ii) = c ; c=c+1 ;
    V0(c) = BC.O8(0) ; ID.PW.O8(ii) = c ; c=c+1 ;
    V0(c) = BC.C2(0) ; ID.PW.C2(ii) = c ; c=c+1 ;
    V0(c) = BC.C3(0) ; ID.PW.C3(ii) = c ; c=c+1 ;
    
    
    %%  Carbonate
    V0(c) = MolConCarb / (1 + INIT.MgCa(ii) + INIT.SrCa(ii)) ; ID.Carb.Ca(ii) = c ; c=c+1 ;
    V0(c) = V0(c-1) * INIT.MgCa(ii) ; ID.Carb.Mg(ii) = c ; c=c+1 ;
    V0(c) = V0(c-2) * INIT.SrCa(ii) ; ID.Carb.Sr(ii) = c ; c=c+1 ;
    V0(c) = MolConCarb * 3 * (1 / (1 + (1+INIT.d18O(ii)/1000)*EQs.ROpdb)) ; ID.Carb.O6(ii) = c ; c=c+1 ;
    V0(c) = V0(c-1) * (1+INIT.d18O(ii)/1000)*EQs.ROpdb ; ID.Carb.O8(ii) = c ; c=c+1 ;
    V0(c) = MolConCarb * (1/(1+(1 + INIT.d13C(ii)/1000)*EQs.RCpdb)); ID.Carb.C2(ii) = c ; c=c+1 ;
    V0(c) = MolConCarb - V0(c-1); ID.Carb.C3(ii) = c ; c=c+1 ;
    
end



tspan = 10.^linspace(4,14,200); % duration of model
out.t = tspan;

options = odeset('maxstep',[10^12]); tic

[out.t,V] = ode15s(@(t,V) OnePhasePDE(t,V,EQs,sys,BC,ID),tspan,V0,options) ;

%% Tidy up Data
out.C.MgCa = V(:,ID.Carb.Mg) ./ (V(:,ID.Carb.Ca) ) ;
out.C.SrCa = V(:,ID.Carb.Sr) ./ (V(:,ID.Carb.Ca) ) ;
out.C.R18O = V(:,ID.Carb.O8) ./ (V(:,ID.Carb.O6) ) ;
out.C.d18O = (out.C.R18O ./ EQs.ROpdb - 1) .* 1000;
out.C.R13C = V(:,ID.Carb.C3) ./ (V(:,ID.Carb.C2) ) ;
out.C.d13C = (out.C.R13C ./ EQs.RCpdb - 1) .* 1000;


out.PW.MgCa = (V(:,ID.PW.Mg)./(V(:,ID.PW.Ca))) ; 
out.PW.SrCa = (V(:,ID.PW.Sr)./(V(:,ID.PW.Ca))) ; 
out.PW.R18O = V(:,ID.PW.O8) ./ (V(:,ID.PW.O6) ) ;
out.PW.d18O = (out.PW.R18O ./ EQs.ROsmow - 1) .* 1000;
out.PW.R13C = V(:,ID.PW.C3) ./ (V(:,ID.PW.C2) ) ;
out.PW.d13C = (out.PW.R13C ./ EQs.RCpdb - 1) .* 1000;

%% Make a few figures showing model output.
figure(1)

set(gcf,'position',[0 1000 600 500])

subplot(2,1,1)
semilogx(out.t,out.C.MgCa)
xlim([10^4 max(tspan)])
hold on
subplot(2,1,2)
semilogx(out.t,out.C.d13C)
xlim([10^4 max(tspan)])
hold on

figure(2)
set(gcf,'position',[600 500 600 500])

plot(out.C.MgCa,out.C.d18O)
hold on
drawnow

%% Save all model output as structure, dat. 
% This structure saves all input and output for this model run.
counter = counter+1;
dat(counter).BC = BC;
dat(counter).EQs = EQs;
dat(counter).ID = ID;
dat(counter).INIT = INIT;
dat(counter).out = out;
dat(counter).sys = sys;
dat(counter).V = V;

end

