clear; close all; clc; format long
counter = 0

%% Rmod and G can be given multiple inputs for multiple model runs.
for Rmod = [100] % Reciprocal reaction rate (here in years)
for G = [1] % G is the relative reaction rate of HMC compared to LMC
    Baker = 0 ;
    
    % Deal with problems from long timesteps (resolves instability in d18O)
    if Rmod <= 10
        maxstep = 10^8 ;
        maxt = 14 ;
    else
        maxstep = 10^11;
        maxt = 14.5;
    end
    
    
    
FLMC = 0.75 ; % Fraction of LMC (rest is HMC)

MgCa0 = 4.49 * 10^-3 ; % Mg/Ca Mean for EPMA values
SrCa0 = 1.46 * 10^-3 ;
d18O0 = -3.523 ; 
d13C0 = +3.33 ;

MgCaGrad = 10 * 10^-3 ; % difference between HMC and LMC components

sys.F1 = FLMC ; % Fraction of F1 (LMC)

sys.dX = 0.00464;

sys.phi = 10^-6 ;

%% sys structure: for parameters for running the model
sys.PR1 = @(t)  1 / (Rmod * 3.154 * 10^7) ; 

sys.PR2 = @(t) sys.PR1(t) * G ;
% sys.PR2 = @(t) 1 / (1 * 3.154 * 10^7) ; % uncomment to override previous line


%% EQs structure: equations for constructing and running model
% Water Information
EQs.PW = 1.03; % Density of water
EQs.AMU.W = 18 ; % AMU of water;

% Carbonate Information
EQs.PC = 2.7; % density of LMC, g/cc

EQs.AMU.C = 100; % AMU of carbonate, g/cc

% Chemistry equations
EQs.alphaO = @(T) exp(( 18030./(T+273.15) - 32.24 ) ./ 1000 ); % Alpha-value for oxygen (Kim and O'Neil, 1997)
EQs.alphaC = @(T) 1.001 ; % Alpha-value for Carbon (Romanek et al., 1992)

if Baker==1
    EQs.Kd.Mg = @(T) 8.1 * 10^-4; % From Baker et al., (1982)
else
    EQs.Kd.Mg = @(T) 0.012 ; % Distribution coefficient for Magnesium Mucci (1987)
end

EQs.Kd.Sr = @(T) 0.05 ; % Distribution coefficient for Strontium Banner (1995) and Banner and Hanson (1990).

% Diffusion coefficients from Boudreu (1997)
EQs.Di.Ca = @(T,phi) (3.60 + 0.144 .* T)*10^-6 ./ (1-log(phi.^2)) ; % Diffusivity of Calcium (T in °C)
EQs.Di.Mg = @(T,phi) (3.43 + 0.144 .* T)*10^-6 ./ (1-log(phi.^2)) ; % Diffusivity of Magnesium
EQs.Di.Sr = @(T,phi) (3.69 + 0.169 .* T)*10^-6 ./ (1-log(phi.^2)) ; % Diffusivity of Strontium
EQs.Di.C  = @(T,phi) (5.06 + 0.275 .* T)*10^-6 ./ (1-log(phi.^2)) ; % Diffusivity of Bicarbonate
EQs.Di.O  = @(T,phi) (8.702 + 0.443 * T + 0.0035 * T^2) * 10^-6 ./ (1-log(phi.^2)) ; % Self-diffusivity of oxygen

EQs.ROsmow = 0.0020052 ; % Baertschi (1976)
EQs.ROpdb = EQs.ROsmow * 1.03092; % PDB Isotope Ratio using SMOW values from Baertschi (1976) and PDB = 1.03092*SMOW from Coplen et al., (1983)
EQs.RCpdb = 0.01118; % Chang and Li (1990)

%% External Boundary Condition
%  Functions in BC are called by time, so they can vary over time.
% Fluid BC values should be in units of mols / g
BC.d18O = @(t) -0.89 ; % d18O of water
BC.DIC = @(t) 1 * 10^-6; % mols / g solvent
BC.d13C = @(t) -0.8; % d13C of Outer DIC

BC.Ca   = @(t) (16 * 10^-6); % Concentration of seawater Ca (mol/g), Horita et al., (2002)
BC.MgCa = @(t) 2.2;  %  Eocene Mg abundance Evans et al., (2018)
BC.SrCa = @(t) 8/1000 ; % Eocene Sr Abundance Lear et al., (2003)

% Calculate things based on above parameters
BC.O6 = @(t) (1 / EQs.AMU.W) * (1 / (1 + (BC.d18O(t)/1000 + 1)*EQs.ROsmow)) ;
BC.O8 = @(t) BC.O6(t) * EQs.ROsmow * (1 + BC.d18O(t)/1000) ;

BC.C2 = @(t) BC.DIC(t) * (1 / (1 + (BC.d13C(t)/1000 + 1) * EQs.RCpdb));
BC.C3 = @(t) BC.DIC(t) - BC.C2(t);

BC.Mg = @(t) BC.Ca(t) * BC.MgCa(t);
BC.Sr = @(t) BC.Ca(t) * BC.SrCa(t);

BC.T = @(t) 10; % temperature (°C)



%% INIT structure: information on initial state of model
INIT.FP1 = sys.F1;
INIT.FP2 = 1-sys.F1;

INIT.d18O = d18O0 ;
INIT.d13C = d13C0 ;

[INIT.MgCa1, INIT.MgCa2] = MixingCalc(MgCa0,SrCa0,MgCaGrad,sys.F1) ; 

if INIT.MgCa1 < 0
    INIT.MgCa1 = 0 ; 
    
end

INIT.SrCa1 = SrCa0;
INIT.SrCa2 = SrCa0;




%% Construct initial vector
c = 1; % c is a counter, keeping track of elements in the initial vector as they are added

MolConCarb = 1/EQs.AMU.C ;  % molar Concentration of Carbonate/g of CaCO3

for ii = 1
    %% Pore Water 
    % Initial porewater is the same as the outer boundary condition at t=0
    V0(c) = BC.Ca(0) ; ID.PW.Ca(ii) = c ; c=c+1 ;
    V0(c) = BC.Mg(0) ; ID.PW.Mg(ii) = c ; c=c+1 ;
    V0(c) = BC.Sr(0) ; ID.PW.Sr(ii) = c ; c=c+1 ;
    V0(c) = BC.O6(0) ; ID.PW.O6(ii) = c ; c=c+1 ;
    V0(c) = BC.O8(0) ; ID.PW.O8(ii) = c ; c=c+1 ;
    V0(c) = BC.C2(0) ; ID.PW.C2(ii) = c ; c=c+1 ;
    V0(c) = BC.C3(0) ; ID.PW.C3(ii) = c ; c=c+1 ;

    
    %% Primary Carbonate 1 (LMC)
    V0(c) = MolConCarb / (1 + INIT.MgCa1(ii) + INIT.SrCa1(ii)) ; ID.PC1.Ca(ii) = c ; c=c+1 ;
    V0(c) = V0(c-1) * INIT.MgCa1(ii) ; ID.PC1.Mg(ii) = c ; c=c+1 ;
    V0(c) = V0(c-2) * INIT.SrCa1(ii) ; ID.PC1.Sr(ii) = c ; c=c+1 ;
    V0(c) = MolConCarb * 3 * (1 / (1 + (1+INIT.d18O(ii)/1000)*EQs.ROpdb)) ; ID.PC1.O6(ii) = c ; c=c+1 ;
    V0(c) = V0(c-1) * (1+INIT.d18O(ii)/1000)*EQs.ROpdb ; ID.PC1.O8(ii) = c ; c=c+1 ;
    V0(c) = MolConCarb * (1/(1+(1 + INIT.d13C(ii)/1000)*EQs.RCpdb)); ID.PC1.C2(ii) = c ; c=c+1 ;
    V0(c) = MolConCarb - V0(c-1); ID.PC1.C3(ii) = c ; c=c+1 ;
    
    %% Primary Carbonate 2 (HMC)
    V0(c) = MolConCarb / (1 + INIT.MgCa2(ii) + INIT.SrCa2(ii)) ; ID.PC2.Ca(ii) = c ; c=c+1 ;
    V0(c) = V0(c-1) * INIT.MgCa2(ii) ; ID.PC2.Mg(ii) = c ; c=c+1 ;
    V0(c) = V0(c-2) * INIT.SrCa2(ii) ; ID.PC2.Sr(ii) = c ; c=c+1 ;
    V0(c) = MolConCarb * 3 * (1 / (1 + (1+INIT.d18O(ii)/1000)*EQs.ROpdb)) ; ID.PC2.O6(ii) = c ; c=c+1 ;
    V0(c) = V0(c-1) * (1+INIT.d18O(ii)/1000)*EQs.ROpdb ; ID.PC2.O8(ii) = c ; c=c+1 ;
    V0(c) = MolConCarb * (1/(1+(1 + INIT.d13C(ii)/1000)*EQs.RCpdb)); ID.PC2.C2(ii) = c ; c=c+1 ;
    V0(c) = MolConCarb - V0(c-1); ID.PC2.C3(ii) = c ; c=c+1 ;
 
end

tspan = 10.^linspace(4,maxt,200);

options = odeset('maxstep',maxstep); tic
out.t = tspan;

[out.t,V] = ode15s(@(t,V) TwoPhaseODE(t,V,EQs,sys,BC,ID),tspan,V0,options) ;

%% Tidy up Data
out.C.MgCa = (V(:,ID.PC1.Mg)*sys.F1 + V(:,ID.PC2.Mg)*(1-sys.F1)) ./ (V(:,ID.PC1.Ca)*sys.F1 + V(:,ID.PC2.Ca)*(1-sys.F1)) ;
out.C.SrCa = (V(:,ID.PC1.Sr)*sys.F1 + V(:,ID.PC2.Sr)*(1-sys.F1)) ./ (V(:,ID.PC1.Ca)*sys.F1 + V(:,ID.PC2.Ca)*(1-sys.F1)) ;
out.C.R18O = (V(:,ID.PC1.O8)*sys.F1 + V(:,ID.PC2.O8)*(1-sys.F1)) ./ (V(:,ID.PC1.O6)*sys.F1 + V(:,ID.PC2.O6)*(1-sys.F1)) ;
out.C.d18O = (out.C.R18O ./ EQs.ROpdb - 1) .* 1000;
out.C.R13C = (V(:,ID.PC1.C3)*sys.F1 + V(:,ID.PC2.C3)*(1-sys.F1)) ./ (V(:,ID.PC1.C2)*sys.F1 + V(:,ID.PC2.C2)*(1-sys.F1)) ;
out.C.d13C = (out.C.R13C ./ EQs.RCpdb - 1) .* 1000;

out.W.MgCa = (V(:,ID.PW.Mg)./(V(:,ID.PW.Ca))) ; 
out.W.SrCa = (V(:,ID.PW.Sr)./(V(:,ID.PW.Ca))) ;
out.W.d18O = (((V(:,ID.PW.O8)./(V(:,ID.PW.O6)))) / EQs.ROsmow - 1 )*1000;
out.W.d13C = (((V(:,ID.PW.C3)./(V(:,ID.PW.C2)))) / EQs.RCpdb - 1 )*1000;

%% Make a few figures showing model output.
figure(1)

set(gcf,'position',[0 1000 600 500])

subplot(2,1,1)
semilogx(out.t,out.C.MgCa)
xlim([10^4 max(tspan)])
hold on
subplot(2,1,2)
semilogx(out.t,out.C.d18O)
xlim([10^4 max(tspan)])
xlabel('time (s)')
ylabel('\delta^{18}O')
hold on

figure(2)
set(gcf,'position',[600 500 600 500])

plot(out.C.MgCa,out.C.d18O)
xlabel('Mg/Ca')
ylabel('\delta^{18}O')
hold on
drawnow

%% Save all model output as structure, dat. 
% This structure saves all input and output for this model run.
counter = counter+1 ;
dat(counter).BC = BC;
dat(counter).EQs = EQs;
dat(counter).ID = ID;
dat(counter).INIT = INIT;
dat(counter).out = out;
dat(counter).sys = sys;
dat(counter).V = V;
end
end